<?xml version="1.0" encoding="UTF-8"?>
<!-- 
    Templates that generate blocks or inlines - targets of named destinations.
    
    By default all id attributes are declared as named destinations.
    
    However, the nd-id attributes are used to generate targets. We needed this support
    for DITA, since the id attributes are rewritten, and the original id value 
    is stored in the nd-id attribute. 
    
    In order to keep working the internal linking, we leave the id attribute untouched, 
    and we generate additional structures for the nd-id attribute. 
        
-->
<xsl:stylesheet 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:fo="http://www.w3.org/1999/XSL/Format" 
    xmlns:css="http://www.w3.org/1998/CSS" 
    xmlns:nd="http://www.oxygenxml.com/css2fo/named-destinations"
    xmlns:fox="http://xmlgraphics.apache.org/fop/extensions"
     
    exclude-result-prefixes="#all" 
    version="2.0">
	<!-- Parameter that enables or disables the generation of named destinations -->
    <xsl:param name="generate-named-destinations"/>

	<!-- Optimization, keep a boolean. -->
    <xsl:variable name="gnd" select="$generate-named-destinations = 'yes'"/>
    
    <xsl:template name="generate-named-destination-target-block">
       <!-- Avoid generating a block or inline before a css:string-set fo:marker  -->
       <xsl:if test="$gnd and @nd:nd-id and not(css:string-set)">
          <fo:block id="{@nd:nd-id}" font-size="0"/>
      </xsl:if>
    </xsl:template>
    
    <xsl:template name="generate-named-destination-target-inline">
       <!-- Avoid generating a block or inline before a css:string-set fo:marker  -->
       <xsl:if test="$gnd and @nd:nd-id and not(css:string-set)">
          <fo:inline id="{@nd:nd-id}" font-size="0"/>
      </xsl:if>
    </xsl:template>

    <!-- It is called from the css:string-set -->
    <xsl:template name="generate-named-destination-target-after-string-set">
      <xsl:choose>
        <xsl:when test="$gnd and ../@nd:nd-id and ../@css:display='inline'">
          <fo:inline id="{../@nd:nd-id}" font-size="0"/>          
        </xsl:when>
        <xsl:when test="$gnd and ../@nd:nd-id ">
          <fo:block id="{../@nd:nd-id}" font-size="0"/>          
        </xsl:when>
      </xsl:choose>
    </xsl:template>
    
    <!-- 
      Dumps the list of fox:destinations for all ids and nd-ids. 
      The nd-id take precedence if id is set on the same element. 
    -->
    <xsl:template name="generate-named-destination-list">
       <xsl:if test="$gnd">
         <xsl:for-each select="//*[@id][not(@nd:nd-id)]" exclude-result-prefixes="fox">
            <fox:destination internal-destination="{@id}"/>
         </xsl:for-each>
         <xsl:for-each select="//*[@nd:nd-id]" exclude-result-prefixes="fox">
            <fox:destination internal-destination="{@nd:nd-id}"/>
         </xsl:for-each>    
       </xsl:if>
    </xsl:template>
    
</xsl:stylesheet>
